package main

import (
	"fmt"
	"log"
	"net/http"
	"os/exec"
	"regexp"
	"runtime"
)

func main() {
	runHandler()
}

func removeANSI(s string) string {
	re := regexp.MustCompile(`\x1b\[[0-?]*[ -/]*[@-~]`)
	return re.ReplaceAllString(s, "")
}

func promptHandler(w http.ResponseWriter, r *http.Request) {
	// get the value of the "prompt" query parameter
	prompt := r.PathValue("prompt")
	log.Println("Prompt: ", prompt)

	// Run ollama prompt
	var cmd = getOllamaCmd(prompt)
	output, err := cmd.CombinedOutput()

	log.Printf("Output: %s, Error: %v", output, err)

	finalOutput := removeANSI(string(output))
	finalOutput = fmt.Sprintf("%s", finalOutput)
	_, err = w.Write([]byte(finalOutput))
	if err != nil {
		log.Println("Error writing response: ", err)
		return
	}
}

func runHandler() {
	http.HandleFunc("/llama/{prompt}", promptHandler)

	log.Println("Starting server on :8080")

	http.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
		_, err := w.Write([]byte("To ask the llama a question, make a GET request to /llama/{prompt}"))
		if err != nil {
			log.Println("Error writing response: ", err)
			return
		}
	})

	log.Fatal(http.ListenAndServe(":8080", nil))
}

func getOllamaCmd(prompt string) *exec.Cmd {
	log.Println("Starting Ollama...")
	if runtime.GOOS == "windows" {
		panic("Unsupported OS")
	} else if runtime.GOOS == "linux" {
		return exec.Command("ollama", "run", "llama3.1", prompt)
	} else {
		panic("Unsupported OS")
	}
}
